@@ -9,8 +9,9 @@ from pydantic import BaseModel, Extra, root_validator, validator
 from yaml import safe_dump, safe_load
 
 from .compat import get_path_group
-from .tools import DummyLocker, DummySize, DummyUsage, Locker, SizeTracker, UsageTracker
-from .utils import PathLike, mkdir
+from .interface import PathOrStr
+from .tools import DummyLocker, DummySize, DummyUsage, Locker, SizeTracker, UsageTracker, LabelsStorage, DummyLabels
+from .utils import mkdir
 
 CONFIG_NAME = 'config.yml'
 
@@ -68,6 +69,7 @@ class StorageConfig(_NoExtra):
     locker: ToolConfig = None
     size: ToolConfig = None
     usage: ToolConfig = None
+    labels: ToolConfig = None
     free_disk_size: Union[int, str] = 0
     max_size: Union[int, str] = None
     version: str = None
@@ -88,11 +90,14 @@ class StorageConfig(_NoExtra):
     def make_usage(self, root: Path) -> UsageTracker:
         return self._make(UsageTracker, DummyUsage, self.usage, root)
 
+    def make_labels(self, root: Path) -> LabelsStorage:
+        return self._make(LabelsStorage, DummyLabels, self.labels, root)
+
     @validator('free_disk_size', 'max_size')
     def to_size(cls, v):
         return parse_size(v)
 
-    @validator('hash', 'locker', 'usage', pre=True)
+    @validator('hash', 'locker', 'usage', 'labels', pre=True)
     def normalize_tools(cls, v):
         if isinstance(v, str):
             v = {'name': v}
@@ -115,7 +120,7 @@ def load_config_buffer(data: str) -> StorageConfig:
     return StorageConfig.parse_obj(safe_load(io.StringIO(data)))
 
 
-def load_config(root: PathLike) -> StorageConfig:
+def load_config(root: PathOrStr) -> StorageConfig:
     with open(Path(root) / CONFIG_NAME) as file:
         return StorageConfig.parse_obj(safe_load(file))
 
@@ -137,7 +142,7 @@ def find_subclass(base, name):
     raise ValueError(f'Could not find a {base.__name__} named {name}')
 
 
-def init_storage(config: StorageConfig, root: PathLike, *,
+def init_storage(config: StorageConfig, root: PathOrStr, *,
                  permissions: Union[int, None] = None, group: Union[str, int, None] = None, exist_ok: bool = False):
     root = Path(root)
     mkdir(root, permissions, group, parents=True, exist_ok=exist_ok)
