@@ -9,8 +9,9 @@ from ..compat import copy_file, remove_file, rmtree
 from ..config import load_config, root_params
 from ..digest import key_to_relative
 from ..exceptions import StorageCorruption
-from ..interface import Key, Keys, MaybeValue, PathOrStr, Value
+from ..interface import Key, Keys, MaybeValue, PathOrStr, Value, MaybeLabels
 from ..tools import Locker, SizeTracker, UsageTracker
+from ..tools.labels import LabelsStorage
 from ..utils import adjust_permissions, create_folders, get_size, match_buffers, match_files
 from .interface import Writable
 
@@ -28,13 +29,16 @@ class DiskDict(Writable):
 
         self.root = root
         self.permissions, self.group = root_params(self.root)
+        # TODO: create these folders only if needed
         usage_folder = self.root / 'tools/usage'
-        # FIXME: race condition
+        labels_folder = self.root / 'tools/labels'
         create_folders(usage_folder, self.permissions, self.group)
+        create_folders(labels_folder, self.permissions, self.group)
 
         self.locker: Locker = config.make_locker()
         self.size_tracker: SizeTracker = config.make_size()
         self.usage_tracker: UsageTracker = config.make_usage(usage_folder)
+        self.labels: LabelsStorage = config.make_labels(labels_folder)
         self.min_free_size = config.free_disk_size
         self.max_size = config.max_size
 
@@ -52,7 +56,7 @@ class DiskDict(Writable):
                 if file.is_dir():
                     file = file / 'data'
 
-                self.usage_tracker.update(key, file)
+                self.usage_tracker.update(key)
                 try:
                     yield file
                     return
@@ -71,7 +75,7 @@ class DiskDict(Writable):
                 yield key, value
 
     @contextmanager
-    def write(self, key: Key, value: Value) -> ContextManager[MaybePath]:
+    def write(self, key: Key, value: Value, labels: MaybeLabels) -> ContextManager[MaybeValue]:
         file = self._key_to_path(key)
         with self.locker.write(key):
             try:
@@ -81,13 +85,9 @@ class DiskDict(Writable):
                     if file.is_dir():
                         file = file / 'data'
 
-                    if _is_pathlike(value):
-                        match_files(value, file)
-                    else:
-                        with open(file, 'rb') as dst:
-                            match_buffers(value, dst, context=key.hex())
-
+                    _match(value, file, key)
                     yield file
+                    self.labels.update(key, labels)
                     return
 
                 # make sure we can write
@@ -99,13 +99,22 @@ class DiskDict(Writable):
                     # create base folders
                     create_folders(file.parent, self.permissions, self.group)
                     # populate the folder
-                    if _is_pathlike(value):
-                        copy_file(value, file)
-                    else:
-                        with open(file, 'wb') as dst:
-                            shutil.copyfileobj(value, dst)
+                    # TODO: make this an atomic operation
+                    #  now this is just an ugly crutch
+                    try:
+                        if _is_pathlike(value):
+                            copy_file(value, file)
+                        else:
+                            with open(file, 'wb') as dst:
+                                shutil.copyfileobj(value, dst)
 
-                    adjust_permissions(file, self.permissions, self.group, read_only=True)
+                    except PermissionError:
+                        if not file.exists():
+                            raise
+                        _match(value, file, key)
+
+                    else:
+                        adjust_permissions(file, self.permissions, self.group, read_only=True)
 
                 except BaseException as e:
                     if file.exists():
@@ -114,7 +123,8 @@ class DiskDict(Writable):
 
                 # metadata
                 self.size_tracker.inc(get_size(file))
-                self.usage_tracker.update(key, file)
+                self.usage_tracker.update(key)
+                self.labels.update(key, labels)
 
                 yield file
 
@@ -139,6 +149,7 @@ class DiskDict(Writable):
 
             self.size_tracker.dec(size)
             self.usage_tracker.delete(key)
+            self.labels.delete(key)
 
             return True
 
@@ -159,3 +170,11 @@ class DiskDict(Writable):
 
 def _is_pathlike(x):
     return isinstance(x, (os.PathLike, str))
+
+
+def _match(value, file, key):
+    if _is_pathlike(value):
+        match_files(value, file)
+    else:
+        with open(file, 'rb') as dst:
+            match_buffers(value, dst, context=key.hex())
